#pragma once

#include <Arduino.h>
#include <SPI.h>
#include <vector>

// Definitions for AD7768 Registers.
#define AD7768_CHSTDB 0x00
#define AD7768_CHMODEA 0x01
#define AD7768_CHMODEB 0x02
#define AD7768_CHMODSEL 0x03
#define AD7768_POWERMODE 0x04
#define AD7768_GENCONF 0x05
#define AD7768_DATACONTROL 0x06
#define AD7768_INTCONF 0x07
#define AD7768_BISTCTL 0x08
#define AD7768_STATUS 0x09
#define AD7768_REVID 0x0A
#define AD7768_GPIOCTL 0x0E
#define AD7768_GPIOWRITE 0x0F
#define AD7768_GPIOREAD 0x10
#define AD7768_PRECHGBUF1 0x11
#define AD7768_PRECHGBUF2 0x12
#define AD7768_POSREFBUF 0x13
#define AD7768_NEGREFBUF 0x14
#define AD7768_DIAGREC 0x56
#define AD7768_DIAGMUX 0x57
#define AD7768_MODDELCTL 0x58
#define AD7768_CHOPCTL 0x59

// Definition of READ/WRITE code to or with the register address.
#define AD7768_WRITE 0x00
#define AD7768_READ 0x80

class AD7768 {
  private:
    float _MCLK;
    byte _CSpin, _STARTpin;
    float _VCM, _DUEScaler;
    SPISettings _AD7768SPISettings;
    byte _dataPins[10];
    unsigned int _rawData[8];
    float _voltages[8];
    void SendCommand(std::vector<byte> &data);
  public:
    AD7768(byte CSpin, byte STARTpin, byte DCLKpin, byte DRDYpin, byte D0pin, byte D1pin, byte D2pin, byte D3pin, byte D4pin, byte D5pin, byte D6pin, byte D7pin);
    void begin(void);
    float getValue(byte channel);
    unsigned int getData(byte channel);
    void writeValue(unsigned int* values);
    byte* getDataPins(void);
    void PulseStart(void);
    float getVCM(void);
    float getDUEScaler(void);
    void calibrate(void);
};

